@@ -3,7 +3,7 @@
 
     const app = angular.module("dataiku.fm.loadbalancers", ["dataiku.services", "dataiku.filters", "dataiku.fm.dialogs"]);
 
-    const managedDNSHostnamePattern = /^[^.]*$/;
+    const managedDNSHostnamePattern = /^[^. ]*$/;
     const unmanagedDNSHostnamePattern = /^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9-]*[A-Za-z0-9])$/;
 
     function _isLoadbalancerValid(loadBalancer) {
@@ -368,6 +368,8 @@
 
 
     app.controller("LoadBalancersSettingsController", function ($scope, $stateParams, LoadBalancersCreationDataService, FMAPI) {
+        $scope.formHolder = {}; // necessary because form is created in a child scope
+
         FMAPI.loadbalancers.get($stateParams.loadBalancerId).success(function (lb) {
             // eslint-disable-next-line no-undef
             $scope.editedLoadBalancer = lb;
@@ -471,16 +473,16 @@
             return result;
         }
 
-        $scope.$watch('loadbalancerSettingsForm.$valid', function (newVal) {
+        $scope.$watch('formHolder.loadbalancerSettingsForm.$valid', function (newVal) {
             $scope.SharedState.loadbalancerFormIsValid = newVal && _isLoadbalancerValid($scope.editedLoadBalancer);
         })
 
-        $scope.$watch('loadbalancerSettingsForm.$dirty', function (newVal) {
+        $scope.$watch('formHolder.loadbalancerSettingsForm.$dirty', function (newVal) {
             $scope.SharedState.loadbalancerFormIsDirty = newVal;
         })
 
         $scope.$watch("editedLoadBalancer", function (nv) {
-            $scope.SharedState.loadbalancerFormIsValid = _isLoadbalancerValid($scope.editedLoadBalancer);
+            $scope.SharedState.loadbalancerFormIsValid = $scope.formHolder.loadbalancerSettingsForm.$valid && _isLoadbalancerValid($scope.editedLoadBalancer);
             if ($scope.loadBalancer && $scope.editedLoadBalancer && nv) {
                 // eslint-disable-next-line no-undef
                 let cleanEdited = dkuDeepCopy($scope.editedLoadBalancer, filterDollarKey);
