@@ -6,7 +6,7 @@
     /**
      * Was previously located here: src/main/platypus/static/dataiku/js/dashboards/edit.js
      */
-    app.controller('ExportDashboardModalController', function($scope, DataikuAPI, ActivityIndicator, FutureProgressModal, WT1, DashboardFilters, DashboardFiltersUrlParams) {
+    app.controller('ExportDashboardModalController', function($scope, DataikuAPI, ActivityIndicator, FutureProgressModal, WT1, DashboardFilters, DashboardFiltersUrlParams, translate) {
         /**
          * @param {Dashboard | undefined} selection when doing mass export, the selection is undefined and
          * $scope.selection.selectedObjects contains the dashboards to export
@@ -19,7 +19,7 @@
             let filtersBySlide = [];
             // It's only possible to take into account filters when from a dashboard in view mode.
             if (options.from === 'VIEW') {
-                filtersBySlide = selection.pages.map(page => DashboardFiltersUrlParams.getFiltersQueryStringValue(DashboardFilters.getActiveGlobalFilters(page.id) || []));
+                filtersBySlide = selection.pages.map(page => DashboardFiltersUrlParams.getFiltersQueryStringValue(DashboardFilters.getActiveGlobalFilters(page.cid) || []));
             }
 
             if (options.massExport == true) {
@@ -33,15 +33,15 @@
                 });
                 if ($scope.selection.selectedObjects.length == 1) {
                     $scope.projectKey = $scope.dashboard.projectKey;
-                    $scope.modalTitle = 'Export dashboard : ' + $scope.dashboard.name;
+                    $scope.modalTitle = translate('DASHBOARD.EXPORT_MODAL.EXPORT_ONE_DASHBOARD', 'Export dashboard : {{dashboardName}}', { dashboardName: $scope.dashboard.name });
                 } else {
                     $scope.projectKey = $scope.selection.selectedObjects.length == 0 ? null : $scope.selection.selectedObjects[0].projectKey;
-                    $scope.modalTitle = 'Export a list of ' + $scope.selection.selectedObjects.length + ' dashboards';
+                    $scope.modalTitle = translate('DASHBOARD.EXPORT_MODAL.EXPORT_MULTIPLE_DASHBOARDS', 'Export a list of {{numberOfDashboards}} dashboards', { numberOfDashboards: $scope.selection.selectedObjects.length });
                 }
             } else {
                 $scope.projectKey = $scope.dashboard.projectKey;
                 $scope.params.dashboards = [ { dashboardId: $scope.dashboard.id, filtersBySlide, slideIndex: undefined } ];
-                $scope.modalTitle = 'Export dashboard : ' + $scope.dashboard.name;
+                $scope.modalTitle = translate('DASHBOARD.EXPORT_MODAL.EXPORT_ONE_DASHBOARD', 'Export dashboard : {{dashboardName}}', { dashboardName: $scope.dashboard.name });
             }
         };
 
@@ -56,12 +56,12 @@
             DataikuAPI.dashboards.export($scope.projectKey, $scope.params.exportFormat, $scope.params.dashboards)
                 .error(setErrorInScope.bind($scope))
                 .success(function(resp) {
-                    FutureProgressModal.show($scope, resp, 'Export dashboard').then(function(result) {
+                    FutureProgressModal.show($scope, resp, translate('DASHBOARD.EXPORT_MODAL.PROGRESS_MODAL', 'Export dashboard')).then(function(result) {
                         if (result) { // undefined in case of abort
                             downloadURL(DataikuAPI.dashboards.getExportURL(result.projectKey, result.exportId));
-                            ActivityIndicator.success('Dashboard export(s) downloaded!', 5000);
+                            ActivityIndicator.success(translate('DASHBOARD.EXPORT_MODAL.PROGRESS_MODAL.SUCCESS', 'Dashboard export(s) downloaded!'), 5000);
                         } else {
-                            ActivityIndicator.error('Export dashboard failed', 5000);
+                            ActivityIndicator.error(translate('DASHBOARD.EXPORT_MODAL.PROGRESS_MODAL.ERROR', 'Export dashboard failed'), 5000);
                         }
                         $scope.resolveModal();
                     });
