@@ -28,10 +28,6 @@ describe('chart features kpi', function () {
         expectClassToBeVisible('qa_charts_measure-dropzone');
     })
 
-    it('doesn\'t have along dimension option', function () {
-        ChartFeaturesTestUtils.computeAlongDimensionSelectTests(chartType, 'NORMAL', false);
-    })
-
     it('doesn\'t have regression line options', function () {
         renderTemplate(`/templates/simple_report/types/options-${chartType}.html`);
         expectIdToBeHidden('qa_charts_regression-line-form');
