@@ -18,26 +18,28 @@ from docling.backend.pdf_backend import PdfDocumentBackend
 from docling.backend.pypdfium2_backend import PyPdfiumDocumentBackend
 from docling.datamodel.base_models import InputFormat, ConversionStatus
 from docling.datamodel.layout_model_specs import DOCLING_LAYOUT_EGRET_MEDIUM, DOCLING_LAYOUT_HERON, DOCLING_LAYOUT_V2, LayoutModelConfig
-from docling.datamodel.pipeline_options import PdfPipelineOptions, TableFormerMode, AcceleratorOptions, AcceleratorDevice, EasyOcrOptions, \
-    PaginatedPipelineOptions, LayoutOptions, ThreadedPdfPipelineOptions
+from docling.datamodel.pipeline_options import TableFormerMode, AcceleratorOptions, AcceleratorDevice, EasyOcrOptions, \
+    PaginatedPipelineOptions, ThreadedPdfPipelineOptions
+from docling.datamodel.settings import settings
 from docling.document_converter import DocumentConverter, PdfFormatOption, WordFormatOption, PowerpointFormatOption, HTMLFormatOption
+from docling.models.document_picture_classifier import DocumentPictureClassifier
 from docling.models.table_structure_model import TableStructureModel
 from docling.pipeline.simple_pipeline import SimplePipeline
-from docling.pipeline.standard_pdf_pipeline import StandardPdfPipeline
 from docling.pipeline.threaded_standard_pdf_pipeline import ThreadedStandardPdfPipeline
 from docling_core.types import DoclingDocument
-from docling_core.types.doc import DocItemLabel, ProvenanceItem, ImageRef, TextItem, PictureItem, GroupItem, TableItem, GroupLabel, ListItem
+from docling_core.types.doc import DocItemLabel, ProvenanceItem, ImageRef, TextItem, PictureItem, GroupItem, TableItem, GroupLabel, ListItem, \
+    PictureClassificationClass
+from docling_core.types.doc.document import PictureClassificationPrediction
 from docling_core.types.io import DocumentStream
 from pydantic import AnyUrl
-from docling.datamodel.settings import settings
-
 
 from dataiku.base.batcher import Batcher
-from dataiku.llm.docextraction.ocr import get_ocr_config, process_images_with_ocr
 from dataiku.doctor.utils.gpu_execution import TorchGpuCapability
+from dataiku.llm.docextraction.ocr import get_ocr_config, process_images_with_ocr
 
 logger = logging.getLogger("docling_extraction")
 
+
 @dataclass
 class DoclingRequest:
     file_name: str
@@ -46,6 +48,7 @@ class DoclingRequest:
     ocr_engine: str
     lang: str
     vlm_annotate: bool
+    save_imgs: bool
 
 @dataclass
 class DoclingResponse:
@@ -63,7 +66,6 @@ class DocumentNode:
     node_type = None
     level: int
 
-
     def __init__(self, node_id: str, children: List, label, content: str = "", level: int = None, page_provenance: List[ProvenanceItem] = None):
         self.node_id = node_id
         self.children = children  # List of children DocumentNode
@@ -89,8 +91,10 @@ class DocumentNode:
 class ImageNode(DocumentNode):
     node_type = "image"
 
-    def __init__(self, node_id, children, label, content="", level=None, page_provenance=None, image_base64=None, image: ImageRef = None, caption=None):
+    def __init__(self, node_id, children, label, highest_confidence_class: Optional[PictureClassificationPrediction], content="", level=None, page_provenance=None,
+                 image_base64=None, image: ImageRef = None, caption=None):
         super().__init__(node_id, children, label, content, level, page_provenance)
+        self.highest_confidence_class = highest_confidence_class
         self.mime_type = image.mimetype if image is not None else None
         self.height = image.size.height if image is not None else None
         self.width = image.size.width if image is not None else None
@@ -118,6 +122,11 @@ class ImageNode(DocumentNode):
             }
         if self.caption:
             res["caption"] = self.caption
+        if self.highest_confidence_class:
+            res["classificationData"] = {"className": self.highest_confidence_class.class_name}
+            if self.highest_confidence_class.confidence is not None:
+                res["classificationData"]["confidence"] = self.highest_confidence_class.confidence
+
         return res
 
 class TextNode(DocumentNode):
@@ -145,6 +154,9 @@ class SectionNode(DocumentNode):
         res["level"] = self.level
         return res
 
+class SlideNode(SectionNode):
+    node_type = "slide"
+
 class RootNode(DocumentNode):
     node_type = "document"
 
@@ -159,6 +171,7 @@ class DoclingExtractorPipeline:
     layout_model: LayoutModelConfig
     artifacts_path: Optional[Path]
     pdf_backend: PdfDocumentBackend = DoclingParseV4DocumentBackend
+    do_image_classification: bool = False
 
     def __init__(self, kernel_settings):
         self.executor = ThreadPoolExecutor()
@@ -172,6 +185,7 @@ class DoclingExtractorPipeline:
             self.config_optimization_mode = kernel_settings.get("configOptimizationMode")
         if "pdfBackend" in kernel_settings:
             self.pdf_backend = PyPdfiumDocumentBackend if kernel_settings.get("pdfBackend").lower() == "pypdfium" else self.pdf_backend
+        self.do_image_classification = kernel_settings.get("enableImageClassificationFiltering", False)
 
         self.device = self.detect_accelerator_device()
 
@@ -181,7 +195,8 @@ class DoclingExtractorPipeline:
         # Pdf extraction with docling requires those IBM models tableformer and layout:  https://huggingface.co/ds4sd/docling-models
         # We'll check if the models are available within the resource folder. If this is the case we'll use them for extraction. If not we'll continue regular
         # execution (check hf cache check + download if not present)
-        self.layout_model, self.artifacts_path = self.find_docling_pdf_models_in_resources(self.table_structure_options_mode)
+        self.layout_model, self.artifacts_path = self.find_docling_pdf_models_in_resources(self.table_structure_options_mode,
+                                                                                           do_image_classification=self.do_image_classification)
         logger.info("Artifact path: %s", self.artifacts_path)
         # Layout Model
         logger.info("Layer model used: %s", self.layout_model.name)
@@ -207,10 +222,10 @@ class DoclingExtractorPipeline:
         return await self.batcher.process(DoclingRequest(process_document_command["fileName"], process_document_command["documentContent"],
                                                          process_document_command.get("imageHandlingMode", "IGNORE") == "OCR",
                                                          ocr_settings.get("ocrEngine", "AUTO"), ocr_settings.get("ocrLanguages", []),
-                                                         process_document_command.get("imageHandlingMode", "IGNORE") == "VLM_ANNOTATE"))
+                                                         process_document_command.get("imageHandlingMode", "IGNORE") == "VLM_ANNOTATE",
+                                                         process_document_command.get("saveImgs", False)))
 
-
-    def find_docling_pdf_models_in_resources(self, table_mode: TableFormerMode) -> (LayoutModelConfig, Optional[Path]):
+    def find_docling_pdf_models_in_resources(self, table_mode: TableFormerMode, do_image_classification: bool) -> (LayoutModelConfig, Optional[Path]):
         """
         Look in the resources folder whether the docling PDF models are available. Returns the path to the directory containing the models (that should be
         passed to docling) or None if one or more models is missing
@@ -259,8 +274,34 @@ class DoclingExtractorPipeline:
         else:
             logger.info(u"No table model found in {}".format(tableformer_path))
 
+        # Image classification model:
+        if do_image_classification:
+            image_classification_model_path = (artifact_path / DocumentPictureClassifier._model_repo_folder)
+            logger.info(u"Searching for image classification model in {}".format(image_classification_model_path))
+            if self.model_exists(image_classification_model_path):
+                logger.info(u"Image classification model found in {}".format(image_classification_model_path))
+                classification_model_found = True
+            else:
+                logger.info(u"No image classification model found in {}".format(image_classification_model_path))
+                classification_model_found = False
+            if layout_model_found and table_model_found and classification_model_found:
+                logger.info(
+                    "Layout, table and image classification Docling models for pdf extraction were found in the resources folder of the code env. They will be used for extraction")
+                return layout_model, artifact_path
+            elif layout_model_found and table_model_found and not classification_model_found:
+                logger.info(
+                    "Layout and table Docling models for pdf extraction were found in the resources folder of the code env. But the image classification model was not found. "
+                    "The image classification will be disabled")
+                self.do_image_classification = False
+                return layout_model, artifact_path
+            else:
+                logger.info(
+                    "Cannot find layout, table and image classification Docling models in the resources folder of the code env. They will be downloaded from HF.")
+                return original_layout_model, None
+
         if layout_model_found and table_model_found:
-            logger.info("Layout and table Docling models for pdf extraction were found in the resources folder of the code env. They will be used for extraction")
+            logger.info(
+                "Layout and table Docling models for pdf extraction were found in the resources folder of the code env. They will be used for extraction")
             return layout_model, artifact_path
         else:  # we cannot set separately the layout and table path but only set the root artifact_path. If one of them is not present, extraction will fail.
             logger.info("Cannot find both layout and table Docling models in the resources folder of the code env. They will be downloaded from HF.")
@@ -288,7 +329,6 @@ class DoclingExtractorPipeline:
         else:
             return DOCLING_LAYOUT_HERON if is_gpu_available else DOCLING_LAYOUT_EGRET_MEDIUM
 
-
     @staticmethod
     def detect_accelerator_device() -> AcceleratorDevice:
         # Detects if GPU is available else fallback on CPU or MPS (macOS).
@@ -322,8 +362,12 @@ class DoclingExtractorPipeline:
 
         needs_ocr = any(req.do_ocr for req in requests)
         vlm_annotate = any(req.vlm_annotate for req in requests)
+        save_images = any(req.save_imgs for req in requests)
         apply_additional_ocr_on_generated_images = False  # For docx/pptx, docling does not apply OCR on generated images, so we need to do it manually if needed.
 
+        # To save images in a managed folder, we need their base64. We only don't need them if the image handling mode is set to ignore.
+        with_inline_images = save_images or vlm_annotate or needs_ocr
+
         if needs_ocr:
             ocr_options = get_ocr_config(requests[0].lang, requests[0].ocr_engine, accelerator_options.device != AcceleratorDevice.CPU)
         else:
@@ -337,9 +381,9 @@ class DoclingExtractorPipeline:
             text_results = process_images_with_ocr(ocr_options, [(idx, request.document_content) for idx, request in enumerate(requests)])
             for idx, request in enumerate(requests):
                 if idx in text_results:
-                    res.append(DoclingResponse(True, ImageNode(request.file_name, [], "image", text_results[idx], 0).to_dict(), None).to_dict())
+                    res.append(DoclingResponse(True, ImageNode(request.file_name, [], "image", None, text_results[idx], 0).to_dict(), None).to_dict())
                 else:
-                    #something went wrong with the OCR processing
+                    # something went wrong with the OCR processing
                     res.append(DoclingResponse(False, None, f"Error processing image {request.file_name} with OCR").to_dict())
             return res
         # handle PDFs
@@ -348,9 +392,10 @@ class DoclingExtractorPipeline:
                 if ocr_options is None:
                     logger.warning("The provided OCR engine is not available, deactivating OCR for extraction")
                 else:
-                    ocr_options.force_full_page_ocr = False # To apply OCR on each image if pdf is not fully scanned. This will ensure we get good results on embedded images.
+                    ocr_options.force_full_page_ocr = False  # To apply OCR on each image if pdf is not fully scanned. This will ensure we get good results on embedded images.
             pipeline_options = ThreadedPdfPipelineOptions(do_table_structure=True, do_ocr=(needs_ocr and ocr_options is not None),
-                                                  ocr_options=(ocr_options if ocr_options is not None else EasyOcrOptions()))  # Can't set ocr_options to None
+                                                          ocr_options=(
+                                                              ocr_options if ocr_options is not None else EasyOcrOptions()))  # Can't set ocr_options to None
 
             pipeline_options.table_structure_options.mode = self.table_structure_options_mode
             pipeline_options.table_structure_options.do_cell_matching = True
@@ -359,10 +404,14 @@ class DoclingExtractorPipeline:
                 pipeline_options.artifacts_path = self.artifacts_path
 
             pipeline_options.layout_options.model_spec = self.layout_model
-
             pipeline_options.accelerator_options = accelerator_options
-            if vlm_annotate:
+
+            if with_inline_images:
+                # The default resolution for images inside PDFs is 72dpi, we set it to 144dpi.
+                # See https://github.com/docling-project/docling/blob/v2.61.1/docling/pipeline/standard_pdf_pipeline.py#L636-L638
+                pipeline_options.images_scale = 2.0
                 pipeline_options.generate_picture_images = True
+
             doc_converter = DocumentConverter(format_options={
                 InputFormat.PDF: PdfFormatOption(
                     pipeline_options=pipeline_options,
@@ -371,6 +420,10 @@ class DoclingExtractorPipeline:
         # Any other documents (docx, pptx, html)
         else:
             pipeline_options = PaginatedPipelineOptions(accelerator_options=accelerator_options)
+            if with_inline_images:
+                # Note that for docx and pptx, picture images will be generated even if pipeline_options.generate_picture_images = False
+                pipeline_options.generate_picture_images = True
+
             # Docling does not support OCR on docx, pptx, html, so we need to extract the images and apply "raw" OCR on them.
             if needs_ocr:
                 if ocr_options is None:
@@ -394,6 +447,9 @@ class DoclingExtractorPipeline:
                 )
             })
 
+        pipeline_options.do_picture_classification = self.do_image_classification
+        logger.info(build_message_log_for_document([request.file_name for request in requests], "Image classification is " +
+                                                   ("enabled" if pipeline_options.do_picture_classification else "disabled")))
         # Do the docling conversion and map output to tree structure
         document_to_process = []
         for document in requests:
@@ -413,7 +469,7 @@ class DoclingExtractorPipeline:
                     res.append(
                         DoclingResponse(True, build_tree_with_outline(conversion_result.document, needs_ocr,
                                                                    apply_additional_ocr_on_generated_images, ocr_options,
-                                                                   vlm_annotate).to_dict(), None).to_dict())
+                                                                   vlm_annotate, with_inline_images=with_inline_images).to_dict(), None).to_dict())
                     logger.info(build_message_log_for_document(conversion_result.document.name, "Done processing document with docling"))
             except Exception as e:
                 logger.exception("An error occurred during docling processing")
@@ -421,7 +477,7 @@ class DoclingExtractorPipeline:
         return res
 
 def build_tree_with_outline(document: DoclingDocument, needs_ocr, apply_additional_ocr_on_generated_images=False,
-                             ocr_options=None, vlm_annotate=False) -> RootNode:
+                             ocr_options=None, vlm_annotate=False, with_inline_images=False) -> RootNode:
     """
     Given a docling conversion result. We build a tree that represent the structure of the documents. The nodes can either be sections, texts, images or tables.
 
@@ -429,7 +485,8 @@ def build_tree_with_outline(document: DoclingDocument, needs_ocr, apply_addition
     :param needs_ocr: Whether OCR should be applied on the document.
     :param apply_additional_ocr_on_generated_images: For docx/pptx, docling does not apply OCR on images, if needed, do it manually on the generated images.
     :param ocr_options: OCR options if additional OCR needs to be applied.
-    :param vlm_annotate: Whether images will be annotated by a LLM in the java. If so we need to return their base64.
+    :param vlm_annotate: Whether images will be annotated by a LLM in the java. If so we need to look if images have captions.
+    :param with_inline_images: Whether the base64 of the images will be added to image nodes.
     :return:
     """
     root = RootNode(document.body.self_ref, [], "root", "", 0)
@@ -464,7 +521,9 @@ def build_tree_with_outline(document: DoclingDocument, needs_ocr, apply_addition
                 handle_section_item(stack, current_section, level, current_text)
 
             elif type(node_item) == GroupItem and node_item.label in [GroupLabel.CHAPTER, GroupLabel.SLIDE]:
-                current_section = SectionNode(node_item.self_ref, [], node_item.label, node_item.name if hasattr(node_item, "name") else "", level, None)
+                # For now docling identifies slide as chapter see https://github.com/docling-project/docling/blob/main/docling/backend/mspowerpoint_backend.py#L331-L335
+                # We should pay attention to any changes for next docling bump, for the moment chapter is only used in mspowerpoint_backend.py
+                current_section = SlideNode(node_item.self_ref, [], node_item.label, increment_slide(node_item.name) if hasattr(node_item, "name") else "", level, None)
                 handle_section_item(stack, current_section, level, current_text)
 
 
@@ -477,7 +536,7 @@ def build_tree_with_outline(document: DoclingDocument, needs_ocr, apply_addition
 
             elif isinstance(node_item, PictureItem):
                 append_pending_nodes_to_tree(stack, current_text)
-                handle_image_item(document, node_item, needs_ocr, vlm_annotate, apply_additional_ocr_on_generated_images, images_text, captions_cref_list, stack, image_idx, level)
+                handle_image_item(document, node_item, needs_ocr, vlm_annotate, with_inline_images, apply_additional_ocr_on_generated_images, images_text, captions_cref_list, stack, image_idx, level)
                 image_idx += 1
             elif isinstance(node_item, TableItem):
                     append_pending_nodes_to_tree(stack, current_text)
@@ -520,13 +579,14 @@ def handle_section_item(stack: List[DocumentNode], current_section: SectionNode,
         stack[-1].children.append(current_section)
     stack.append(current_section)
 
-def handle_image_item(document, image_item, needs_ocr, vlm_annotate, apply_additional_ocr_on_generated_images, images_text, captions_cref_list, stack, image_idx, level):
+def handle_image_item(document, image_item: PictureItem, needs_ocr, vlm_annotate, with_inline_images, apply_additional_ocr_on_generated_images, images_text, captions_cref_list, stack, image_idx, level):
     """
     Handle an image item from docling conversion. Depending on the settings, it can extract OCR text from the image, or retrieve its base64 representation for VLM annotation (later in Java).
     :param document: docling document
     :param image_item: current image item
     :param needs_ocr: whether OCR is needed
     :param vlm_annotate: whether VLM annotation is needed
+    :param with_inline_images: whether the base64 should be returned
     :param apply_additional_ocr_on_generated_images: Docling only supports OCR on pdfs. For docx/pptx, docling does not apply OCR on images, if needed, we do it manually on the generated images.
     :param images_text: A dictionary of image cref to OCR text for images that needed additional OCR
     :param captions_cref_list:     # Captions for images are returned as part of ImageNode when doing vlm annotation, we keep track of them to avoid duplicates. Because document.iterate_items() will also return them
@@ -536,8 +596,23 @@ def handle_image_item(document, image_item, needs_ocr, vlm_annotate, apply_addit
     :return:
     """
     content = ""
-    image_base64 = None
+    if (
+            isinstance(image_item.image, ImageRef)
+            and isinstance(image_item.image.uri, AnyUrl)
+            and image_item.image.uri.scheme == "data"
+    ) and with_inline_images:
+        image_base64 = image_item.image.uri.path.split(",")[1]
+    else:
+        image_base64 = None
+
     caption = None
+    highest_confidence_class: Optional[PictureClassificationPrediction] = None
+    if not image_item.meta and image_item.annotations:
+        # In docling 2.61.1, annotations are deprecated but meta field is not yet filled... We migrate them to avoid any future issues when annotations are removed.
+        image_item._migrate_annotations_to_meta()
+    if image_item.meta and image_item.meta.classification:
+        highest_confidence_class: PictureClassificationPrediction = image_item.meta.classification.get_main_prediction()
+
     if needs_ocr:
         if not apply_additional_ocr_on_generated_images:
             # OCR was applied on the document, so we can look for text nodes inside the image.
@@ -556,19 +631,17 @@ def handle_image_item(document, image_item, needs_ocr, vlm_annotate, apply_addit
                 # Retrieve the OCR text for the image that we computed at the beginning of the function
                 content = images_text[image_item.self_ref]
     elif vlm_annotate:
-        if (
-                isinstance(image_item.image, ImageRef)
-                and isinstance(image_item.image.uri, AnyUrl)
-                and image_item.image.uri.scheme == "data"
-        ):
-            image_base64 = image_item.image.uri.path.split(",")[1]
+        if image_base64 is not None:
             # We look for caption to enrich the image Node
-            # This caption will also be returned on the iterate_items
+            # This caption will also be returned on the iterate_items, so we need to keep track of them in order not to add them twice
             if image_item.captions:
+                logger.info("trying to find caption for image:")
                 caption = find_captions_for_image(document, image_item)
                 captions_cref_list.extend([ref_item.cref for ref_item in image_item.captions])
     stack[-1].children.append(
-        ImageNode("image_" + str(image_idx), [], image_item.label, content, level, image_item.prov if hasattr(image_item, "prov") else None, image_base64=image_base64,
+        ImageNode("image_" + str(image_idx), [], image_item.label, highest_confidence_class, content, level,
+                  image_item.prov if hasattr(image_item, "prov") else None,
+                  image_base64=image_base64,
                   image=image_item.image if hasattr(image_item, "image") else None, caption=caption))
 
 def find_captions_for_image(document: DoclingDocument, image_node: PictureItem) -> (Optional[str]):
@@ -608,3 +681,14 @@ def merge_paragraphs(current_text: List[TextNode]) -> TextNode:
 
 def build_message_log_for_document(source_file_path, message):
     return f"[{source_file_path}] - {message}"
+
+def increment_slide(slide_name:str) -> str:
+    # Docling slide index start from 0 which introduces a mismatch with the pageRange
+    # We increment the slide-xx, to start at slide-1
+    split_name = slide_name.split('-')
+    if len(split_name) == 2 and split_name[1].isdigit():
+        prefix, idx = split_name
+        return f"{prefix}-{int(idx) + 1}"
+    else:
+        # If the slide name does not follow the expected pattern, just return the name provided by docling
+        return slide_name
